/*
     This file is part of GNUnet.
     (C) 2005 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file src/upload.c
 * @brief code for uploading with gnunet-gtk
 * @author Christian Grothoff
 */

#include "platform.h"
#include "search.h"
#include "helper.h"
#include "main.h"
#include <extractor.h>

#ifdef MINGW
	#include <shlobj.h>
	#ifndef BIF_NONEWFOLDERBUTTON
		#define BIF_NONEWFOLDERBUTTON 0x200
	#endif
#endif


static GtkTreeStore * summary;

/**
 * XML tree for the meta-data dialog of upload.
 * (there can only be one at a time; 
 * maybe NULL at times where there is no dialog)
 */
static GladeXML * metaXML;

/**
 */
void displayUploadUpdate(const char * mainName,
			 const char * filename,
			 unsigned long long completed,
			 unsigned long long total) {
  GtkTreeIter iter;
  GtkTreeIter child;
  int progress;
  char * name;

  if (total != 0)
    progress = 100 * completed / total;
  else
    progress = 100;

  if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(summary),
				    &iter)) {
    do {	
      gtk_tree_model_get(GTK_TREE_MODEL(summary),
			 &iter,
			 UPLOAD_FILENAME, &name,
			 -1);
      if (0 == strcmp(name, filename)) {
	gtk_tree_store_set(GTK_TREE_STORE(summary),
			   &iter,
			   UPLOAD_PROGRESS, progress,
			   -1);
	return;     
      }
      if (0 == strcmp(name, mainName)) {
	if (gtk_tree_model_iter_children(GTK_TREE_MODEL(summary),
					 &child,
					 &iter)) {
	  do {	
	    gtk_tree_model_get(GTK_TREE_MODEL(summary),
			       &child,
			       UPLOAD_FILENAME, &name,
			       -1);
	    if (0 == strcmp(name, filename)) {
	      gtk_tree_store_set(GTK_TREE_STORE(summary),
				 &child,
				 UPLOAD_PROGRESS, progress,
				 -1);
	      return;
	    }
	  } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(summary),
					    &child));	  
	}
	gtk_tree_store_append(GTK_TREE_STORE(summary),
			      &child,
			      &iter);
	gtk_tree_store_set(GTK_TREE_STORE(summary),
			   &child,
			   UPLOAD_FILENAME, filename,
			   UPLOAD_PROGRESS, progress,
			   UPLOAD_URISTRING, NULL,
			   -1);
	return;
      }    
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(summary),
				      &iter));
  }
  if (0 != strcmp(mainName,
		  filename)) {
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &iter,
			  NULL);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &iter,
		       UPLOAD_FILENAME, mainName,
		       UPLOAD_PROGRESS, 0,
		       UPLOAD_URISTRING, NULL,
		       -1);
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &child,
			  &iter);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &child,
		       UPLOAD_FILENAME, filename,
		       UPLOAD_PROGRESS, progress,
		       UPLOAD_URISTRING, NULL,
		       -1);
  } else {
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &iter,
			  NULL);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &iter,
		       UPLOAD_FILENAME, filename,
		       UPLOAD_PROGRESS, progress,
		       UPLOAD_URISTRING, NULL,
		       -1);
  }
}

/**
 */
void displayUploadComplete(const char * mainName,
			   const char * filename,
			   const struct ECRS_URI * uri) {
  GtkTreeIter iter;
  GtkTreeIter child;
  char * name;
  char * us;

  us = ECRS_uriToString(uri);

  if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(summary),
				    &iter)) {
    do {	
      gtk_tree_model_get(GTK_TREE_MODEL(summary),
			 &iter,
			 UPLOAD_FILENAME, &name,
			 -1);
      if (0 == strcmp(name, filename)) {
	gtk_tree_store_set(GTK_TREE_STORE(summary),
			   &iter,
			   UPLOAD_PROGRESS, 100,
			   UPLOAD_URISTRING, us,
			   -1);
	FREE(us);
	return;     
      }
      if (0 == strcmp(name, mainName)) {
	if (gtk_tree_model_iter_children(GTK_TREE_MODEL(summary),
					 &child,
					 &iter)) {
	  do {	
	    gtk_tree_model_get(GTK_TREE_MODEL(summary),
			       &child,
			       UPLOAD_FILENAME, &name,
			       -1);
	    if (0 == strcmp(name, filename)) {
	      gtk_tree_store_set(GTK_TREE_STORE(summary),
				 &child,
				 UPLOAD_PROGRESS, 100,
				 UPLOAD_URISTRING, us,
				 -1);
	      FREE(us);
	      return;
	    }
	  } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(summary),
					    &child));	  
	}
	gtk_tree_store_append(GTK_TREE_STORE(summary),
			      &child,
			      &iter);
	gtk_tree_store_set(GTK_TREE_STORE(summary),
			   &child,
			   UPLOAD_FILENAME, filename,
			   UPLOAD_PROGRESS, 100,
			   UPLOAD_URISTRING, us,
			   -1);
	FREE(us);
	return;
      }    
    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(summary),
				      &iter));
  }
  if (0 != strcmp(mainName,
		  filename)) {
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &iter,
			  NULL);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &iter,
		       UPLOAD_FILENAME, mainName,
		       UPLOAD_PROGRESS, 0,
		       UPLOAD_URISTRING, NULL,
		       -1);
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &child,
			  &iter);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &iter,
		       UPLOAD_FILENAME, filename,
		       UPLOAD_PROGRESS, 100,
		       UPLOAD_URISTRING, NULL,
		       -1);
  } else {
    gtk_tree_store_append(GTK_TREE_STORE(summary),
			  &iter,
			  NULL);
    gtk_tree_store_set(GTK_TREE_STORE(summary),
		       &iter,
		       UPLOAD_FILENAME, filename,
		       UPLOAD_PROGRESS, 100,
		       UPLOAD_URISTRING, us,
		       -1);
  }
  FREE(us);
}


static int publishMetaData(EXTRACTOR_KeywordType type,
			   const char * data,
			   GtkListStore * store) {
  const char * stype;
  GtkTreeIter iter;

  stype = EXTRACTOR_getKeywordTypeAsString(type);
  gtk_list_store_append(store,
			&iter);
  gtk_list_store_set(store,
		     &iter,
		     META_TYPE, type,
		     META_STYPE, stype,
		     META_VALUE, data,
		     -1);
  return OK;
}

static int publishKeyword(const char * data,
			  GtkListStore * store) {
  GtkTreeIter iter;

  gtk_list_store_append(store,
			&iter);
  gtk_list_store_set(store,
		     &iter,
		     0, data,
		     -1);
  return OK;
}

void on_selectAlternativePreviewButton_selection_changed(GtkWidget * preview,
							 GtkWidget * fileChooser) {
  char * fn;

  fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileChooser));
  if (fn == NULL) {
    gtk_image_set_from_pixbuf(GTK_IMAGE(preview),
			      NULL);
  } else {
    GdkPixbuf * buf;

    buf = gdk_pixbuf_new_from_file(fn,
				   NULL);
    gtk_image_set_from_pixbuf(GTK_IMAGE(preview),
			      buf);
    g_object_unref(buf);
    free(fn);
  }
}

void on_metaDataDialogKeywordAddButton_clicked(gpointer dummy,
					       GtkWidget * uploadButton) {
  GtkWidget * keywordList;
  GtkWidget * entryLine;
  const char * keyword;
  GtkListStore * keymodel;
  GtkTreeIter iter;
  
  keywordList = glade_xml_get_widget(metaXML,
				     "metaDataDialogKeywordList");
  keymodel
    = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(keywordList)));

  entryLine = glade_xml_get_widget(metaXML,
				   "fileInformationKeywordEntry"); 
  keyword = gtk_entry_get_text(GTK_ENTRY(entryLine));
  gtk_list_store_append(keymodel,
			&iter);
  gtk_list_store_set(keymodel,
		     &iter,
		     0, keyword,
		     -1); 
  gtk_entry_set_text(GTK_ENTRY(entryLine), "");
}

void on_metaDataDialogMetaDataAddButton_clicked(gpointer dummy,
					       GtkWidget * uploadButton) {
  GtkWidget * metaList;
  GtkWidget * entryLine;
  GtkWidget * typeCB;
  const char * value;
  EXTRACTOR_KeywordType type;
  GtkListStore * metamodel;
  GtkListStore * typemodel;
  GtkTreeIter iter;
  char * stype;
  
  metaList = glade_xml_get_widget(metaXML,
				  "metaDataDialogMetaDataList");
  metamodel
    = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(metaList)));

  entryLine = glade_xml_get_widget(metaXML,
				   "metaDataDialogValueEntry"); 
  value = gtk_entry_get_text(GTK_ENTRY(entryLine));
  typeCB = glade_xml_get_widget(metaXML,
				"metaDataDialogMetaTypeComboBox"); 
  typemodel
    = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(typeCB)));
  if (! gtk_combo_box_get_active_iter(GTK_COMBO_BOX(typeCB),
				      &iter))
    return; /* oops */
  gtk_tree_model_get(GTK_TREE_MODEL(typemodel),
		     &iter,
		     META_STYPE, &stype,
		     META_TYPE, &type,
		     -1);  
  gtk_list_store_append(metamodel,
			&iter);
  gtk_list_store_set(metamodel,
		     &iter,
		     META_TYPE, type,
		     META_STYPE, stype,
		     META_VALUE, value,
		     -1); 
  gtk_entry_set_text(GTK_ENTRY(entryLine), "");
}

void on_fsinsertuploadbutton_clicked(gpointer dummy,
				     GtkWidget * uploadButton) {
  const char * filename;
  GtkWidget * uploadLine;
  GtkWidget * metaList;
  GtkWidget * keywordList;
  GtkWidget * entry;
  GtkWidget * dialog;
  GtkWidget * preview;
  GtkWidget * metaType;
  GtkWidget * recBut;
  GtkWidget * idxBut;
  GtkListStore * metamodel;
  GtkListStore * keymodel;
  GtkCellRenderer * renderer;
  GtkListStore * keywordTypeModel;
  GtkTreeIter iter;
  struct ECRS_MetaData * meta;
  EXTRACTOR_ExtractorList * extractors;
  EXTRACTOR_KeywordType type;
  char * config;
  struct ECRS_URI * keywordURI;
  struct ECRS_URI * gkeywordURI;
  const char * stype;
  GdkPixbuf * pixbuf;
  GdkPixbufLoader * loader;
  unsigned char * thumb;
  size_t ts;
  char * mvalue;
  char ** keywords;
  unsigned int kpos;
  unsigned int ksize;
  const char * filenamerest;

  extractors = EXTRACTOR_loadDefaultLibraries();
  config = getConfigurationString("FS",
				  "EXTRACTORS");
  if (config != NULL) {
    extractors = EXTRACTOR_loadConfigLibraries(extractors,
					       config);
    FREE(config);
  }
    
  uploadLine = glade_xml_get_widget(mainXML,
				    "uploadFilenameComboBoxEntry");
  entry = gtk_bin_get_child(GTK_BIN(uploadLine));
  filename = gtk_entry_get_text(GTK_ENTRY(entry));
  
  metaXML
    = glade_xml_new(gladeFile,
		    "metaDataDialog",
		    NULL);
  glade_xml_signal_autoconnect(metaXML);
  dialog = glade_xml_get_widget(metaXML,
				"metaDataDialog");
  metamodel
    = gtk_list_store_new(META_NUM,
			 G_TYPE_INT,
			 G_TYPE_STRING,
			 G_TYPE_STRING);
  metaList = glade_xml_get_widget(metaXML,
				  "metaDataDialogMetaDataList");
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(metaList),
					      -1,
					      _("Category"),
					      renderer,
					      "text", META_STYPE,
					      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(metaList),
					      -1,
					      _("Value"),
					      renderer,
					      "text", META_VALUE,
					      NULL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(metaList),
			  GTK_TREE_MODEL(metamodel));


  keymodel
    = gtk_list_store_new(1,
			G_TYPE_STRING);
  keywordList = glade_xml_get_widget(metaXML,
				     "metaDataDialogKeywordList");
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(keywordList),
					      -1,
					      _("Keyword"),
					      renderer,
					      "text", 0,
					      NULL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(keywordList),
			  GTK_TREE_MODEL(keymodel));
  meta = ECRS_createMetaData();
  ECRS_extractMetaData(meta,
		       filename,
		       extractors);
  filenamerest = &filename[strlen(filename)-1];
  while ( (filenamerest > filename) &&
	  (filenamerest[-1] != DIR_SEPARATOR) )
    filenamerest--;
  ECRS_addToMetaData(meta,
		     EXTRACTOR_FILENAME,
		     filenamerest);
  ECRS_getMetaData(meta,
		   (ECRS_MetaDataIterator) &publishMetaData,
		   metamodel);
  keywordURI = ECRS_metaDataToUri(meta);
  ECRS_getKeywordsFromUri(keywordURI,
			  (ECRS_KeywordIterator) &publishKeyword,
			  keymodel);
  keywordTypeModel
    = gtk_list_store_new(KTYPE_NUM,
			 G_TYPE_STRING,
			 G_TYPE_INT);
  metaType = glade_xml_get_widget(metaXML,
				  "metaDataDialogMetaTypeComboBox");
  gtk_combo_box_set_model(GTK_COMBO_BOX(metaType),
			  GTK_TREE_MODEL(keywordTypeModel));
  for (type=0;type<EXTRACTOR_getHighestKeywordTypeNumber();type++) {   
    stype = EXTRACTOR_getKeywordTypeAsString(type);    
    gtk_list_store_append(keywordTypeModel,
			  &iter);
    gtk_list_store_set(keywordTypeModel,
		       &iter,
		       KTYPE_STRING, stype,
		       KTYPE_TYPE, type,
		       -1); 
  }
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(metaType),
			     renderer,
			     FALSE);
  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(metaType),
				renderer,
				"text", KTYPE_STRING);
  gtk_combo_box_set_active(GTK_COMBO_BOX(metaType),
			   0);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog),
				  GTK_RESPONSE_OK);
  thumb = NULL;
  ts = ECRS_getThumbnailFromMetaData(meta,
				     &thumb);
  ECRS_freeMetaData(meta);
  ECRS_freeUri(keywordURI);
  preview = glade_xml_get_widget(metaXML,
				 "previewImage");
  if (ts != 0) {
    loader = gdk_pixbuf_loader_new();
    gdk_pixbuf_loader_write(loader,
			    (const guchar*) thumb,
			    ts,
			    NULL);    
    pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
    gdk_pixbuf_loader_close(loader,
			    NULL);    
    gtk_image_set_from_pixbuf(GTK_IMAGE(preview),
			      pixbuf);
    g_object_unref(pixbuf);
  }
  if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_CANCEL) {
    int doIndex;
    int doRec;
    
    recBut = glade_xml_get_widget(mainXML,
				  "scopeRecursiveButton");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(recBut)))
      doRec = YES;
    else
      doRec = NO;
    idxBut = glade_xml_get_widget(mainXML,
				  "indexbutton");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(idxBut)))
      doIndex = YES;
    else
      doIndex = NO;

    meta = ECRS_createMetaData();
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(metamodel),
				      &iter)) {
      do {	
	gtk_tree_model_get(GTK_TREE_MODEL(metamodel),
			   &iter,
			   META_TYPE, &type,
			   META_VALUE, &mvalue,
			   -1);
	ECRS_addToMetaData(meta,
			   type,
			   mvalue);
      } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(metamodel),
					&iter));
    }
    if (GTK_IMAGE_PIXBUF == 
	gtk_image_get_storage_type(GTK_IMAGE(preview))) {
      size_t length;
      char * thumb;

      pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(preview));
      if (pixbuf != NULL) {
	thumb = NULL;
	if (gdk_pixbuf_save_to_buffer(pixbuf,
				      &thumb,
				      &length,
				      "png",
				      NULL,
				      NULL)) {
	  char * binary;
	  size_t pos;
	  size_t end;
	  size_t wpos;
	  size_t i;
	  unsigned int markers[8]; /* 256 bits */
	  unsigned char marker;
	  
	  /* encode! */
	  binary = MALLOC(2 + length + (length+256) / 254);	  
	  pos = 0;
	  wpos = 0;
	  while (pos < length) {
	    /* find unused value between 1 and 255 in
	       the next 254 bytes */
	    end = pos + 254;
	    if (end < pos)
	      break; /* integer overflow! */
	    if (end > length)
	      end = length;
	    memset(markers, 0, sizeof(markers));
	    for (i=pos;i<end;i++)
	      markers[thumb[i]&7] |= 1 << (thumb[i] >> 3);
	    marker = 1;
	    while (markers[marker&7] & (1 << (marker >> 3))) {
	      marker++;
	      GNUNET_ASSERT(marker != 0);
	    }
	    /* recode */
	    binary[wpos++] = marker;
	    for (i=pos;i<end;i++)
	      binary[wpos++] = thumb[i] == 0 ? marker : thumb[i];
	    pos = end;
	  }
	  binary[wpos++] = 0; /* 0-termination! */
	  free(thumb);
	  ECRS_addToMetaData(meta,
			     EXTRACTOR_THUMBNAIL_DATA,
			     binary);
	  FREE(binary);	
	}
      }
    }
    keywords = NULL;
    ksize = 0;
    GROW(keywords,
	 ksize,
	 64);
    kpos = 0;
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(keymodel),
				      &iter)) {
      do {	
	gtk_tree_model_get(GTK_TREE_MODEL(keymodel),
			   &iter,
			   0, &mvalue,
			   -1);
	keywords[kpos++] = mvalue;
	if (kpos == ksize)
	  GROW(keywords,
	       ksize,
	       kpos*2);
      } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(keymodel),
					&iter));
    }
    keywords[kpos] = NULL;

    keywordURI = ECRS_keywordsToUri((const char**)keywords);    
    while (kpos > 0) 
      FREE(keywords[--kpos]);
    GROW(keywords,
	 ksize,
	 0);
  
    if (doRec) {
      gkeywordURI = ECRS_stringToUri(ECRS_URI_PREFIX
				     ECRS_SEARCH_INFIX);      
      FSUI_uploadAll(ctx,
		     filename,
		     getConfigurationInt("FS",
					 "ANONYMITY"), /* FIXME */
		     doIndex,
		     meta,
		     gkeywordURI,
		     keywordURI);
    } else {
      FSUI_upload(ctx,
		  filename,
		  getConfigurationInt("FS",
				      "ANONYMITY"), /* FIXME */
		  doIndex,
		  NO,
		  meta,
		  keywordURI);
    }
    ECRS_freeMetaData(meta);
    ECRS_freeUri(keywordURI);
  }
  gtk_widget_destroy (dialog);
  UNREF(metaXML);
  metaXML = NULL;
}

static char *selectFile() {
#ifndef MINGW
  GtkWidget *dialog;
  GladeXML * uploadXML;
  GtkWidget * recBut;
  char *ret;

  uploadXML
    = glade_xml_new(gladeFile,
		    "uploadfilechooserdialog",
		    NULL);
  glade_xml_signal_autoconnect(uploadXML);
  dialog = glade_xml_get_widget(uploadXML,
				"uploadfilechooserdialog");

  recBut = glade_xml_get_widget(mainXML,
				"scopeRecursiveButton");
  
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(recBut)))
    gtk_file_chooser_set_action(GTK_FILE_CHOOSER(dialog),
				GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
  
  if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_CANCEL) {
    char * filename;
    
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    ret = STRDUP(filename);
    g_free(filename);
  } else
    ret = NULL;

  gtk_widget_destroy (dialog);
  UNREF(uploadXML);
  return ret;
#else /* MINGW */
	return plibc_ChooseDir(_("Choose the file or directory you want to publish."),
		BIF_BROWSEINCLUDEFILES | BIF_USENEWUI | BIF_SHAREABLE | BIF_NONEWFOLDERBUTTON);
#endif /* MINGW */
}

void on_mainFileSharingInsertBrowseButton_clicked(GtkWidget * browseButton,
						  gpointer dummy) {
	char *filename;
  GtkWidget * uploadLine;
  GtkWidget * entry;
  GtkListStore * model;
  GtkTreeIter iter;
  
  if ((filename = selectFile())) {
    uploadLine = glade_xml_get_widget(mainXML,
				      "uploadFilenameComboBoxEntry");
    entry = gtk_bin_get_child(GTK_BIN(uploadLine));
    gtk_entry_set_text(GTK_ENTRY(entry),
		       filename);
    model = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(uploadLine)));
    gtk_list_store_prepend(model,
			   &iter);
    gtk_list_store_set(model,
		       &iter,
		       0,
		       filename,
		       -1);
    free(filename);
  }
}


void fs_upload_start() {
  GtkWidget * uploadList;
  GtkWidget * uploadEntry;
  GtkCellRenderer * renderer;
  GtkListStore * model;

  uploadList = glade_xml_get_widget(mainXML,
				    "activeUploadsList");
  summary =
    gtk_tree_store_new(UPLOAD_NUM,
		       G_TYPE_STRING, /* filename */
		       G_TYPE_INT,    /* progress */
		       G_TYPE_STRING);  /* URI (as string) - after completion */
  gtk_tree_view_set_model(GTK_TREE_VIEW(uploadList),
			  GTK_TREE_MODEL(summary));
  renderer = gtk_cell_renderer_progress_new();  
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(uploadList),
					      -1,
					      _("Filename"),
					      renderer,
					      "text", UPLOAD_FILENAME,
					      "value", UPLOAD_PROGRESS,
					      NULL);
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(uploadList),
					      -1,
					      _("URI"),
					      renderer,
					      "text", UPLOAD_URISTRING,
					      NULL);


  uploadEntry
    = glade_xml_get_widget(mainXML,
			   "uploadFilenameComboBoxEntry");

  model = gtk_list_store_new(1, G_TYPE_STRING);
  gtk_combo_box_set_model(GTK_COMBO_BOX(uploadEntry),
			  GTK_TREE_MODEL(model));
  gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(uploadEntry),
				      0);
}

void fs_upload_stop() {
}

/* end of upload.c */
